@@ -9,17 +9,28 @@ ERROR_OUTPUT_MAX_LINE_COUNT = 25
 BORG_ERROR_EXIT_CODE = 2
 
 
-def borg_command(full_command):
+def exit_code_indicates_error(command, exit_code, error_on_warnings=False):
     '''
-    Return True if this is a Borg command, or False if it's some other command.
+    Return True if the given exit code from running the command corresponds to an error.
     '''
-    return 'borg' in full_command[0]
+    # If we're running something other than Borg, treat all non-zero exit codes as errors.
+    if 'borg' in command[0] and not error_on_warnings:
+        return bool(exit_code >= BORG_ERROR_EXIT_CODE)
+
+    return bool(exit_code != 0)
 
 
-def execute_and_log_output(full_command, output_log_level, shell, environment):
+def execute_and_log_output(
+    full_command, output_log_level, shell, environment, working_directory, error_on_warnings
+):
     last_lines = []
     process = subprocess.Popen(
-        full_command, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=shell, env=environment
+        full_command,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.STDOUT,
+        shell=shell,
+        env=environment,
+        cwd=working_directory,
     )
 
     while process.poll() is None:
@@ -41,13 +52,7 @@ def execute_and_log_output(full_command, output_log_level, shell, environment):
 
     exit_code = process.poll()
 
-    # If we're running something other than Borg, treat all non-zero exit codes as errors.
-    if borg_command(full_command):
-        error = bool(exit_code >= BORG_ERROR_EXIT_CODE)
-    else:
-        error = bool(exit_code != 0)
-
-    if error:
+    if exit_code_indicates_error(full_command, exit_code, error_on_warnings):
         # If an error occurs, include its output in the raised exception so that we don't
         # inadvertently hide error output.
         if len(last_lines) == ERROR_OUTPUT_MAX_LINE_COUNT:
@@ -59,13 +64,19 @@ def execute_and_log_output(full_command, output_log_level, shell, environment):
 
 
 def execute_command(
-    full_command, output_log_level=logging.INFO, shell=False, extra_environment=None
+    full_command,
+    output_log_level=logging.INFO,
+    shell=False,
+    extra_environment=None,
+    working_directory=None,
+    error_on_warnings=False,
 ):
     '''
     Execute the given command (a sequence of command/argument strings) and log its output at the
     given log level. If output log level is None, instead capture and return the output. If
     shell is True, execute the command within a shell. If an extra environment dict is given, then
-    use it to augment the current environment, and pass the result into the command.
+    use it to augment the current environment, and pass the result into the command. If a working
+    directory is given, use that as the present working directory when running the command.
 
     Raise subprocesses.CalledProcessError if an error occurs while running the command.
     '''
@@ -73,22 +84,34 @@ def execute_command(
     environment = {**os.environ, **extra_environment} if extra_environment else None
 
     if output_log_level is None:
-        output = subprocess.check_output(full_command, shell=shell, env=environment)
+        output = subprocess.check_output(
+            full_command, shell=shell, env=environment, cwd=working_directory
+        )
         return output.decode() if output is not None else None
     else:
-        execute_and_log_output(full_command, output_log_level, shell=shell, environment=environment)
+        execute_and_log_output(
+            full_command,
+            output_log_level,
+            shell=shell,
+            environment=environment,
+            working_directory=working_directory,
+            error_on_warnings=error_on_warnings,
+        )
 
 
-def execute_command_without_capture(full_command):
+def execute_command_without_capture(full_command, working_directory=None, error_on_warnings=False):
     '''
     Execute the given command (a sequence of command/argument strings), but don't capture or log its
     output in any way. This is necessary for commands that monkey with the terminal (e.g. progress
     display) or provide interactive prompts.
+
+    If a working directory is given, use that as the present working directory when running the
+    command.
     '''
     logger.debug(' '.join(full_command))
 
     try:
-        subprocess.check_call(full_command)
+        subprocess.check_call(full_command, cwd=working_directory)
     except subprocess.CalledProcessError as error:
-        if error.returncode >= BORG_ERROR_EXIT_CODE:
+        if exit_code_indicates_error(full_command, error.returncode, error_on_warnings):
             raise
